//%LICENSE////////////////////////////////////////////////////////////////
//
// Licensed to The Open Group (TOG) under one or more contributor license
// agreements.  Refer to the OpenPegasusNOTICE.txt file distributed with
// this work for additional information regarding copyright ownership.
// Each contributor licenses this file to you under the OpenPegasus Open
// Source License; you may not use this file except in compliance with the
// License.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//////////////////////////////////////////////////////////////////////////
//
//%/////////////////////////////////////////////////////////////////////////////

#ifndef Pegasus_SubscriptionKey_h
#define Pegasus_SubscriptionKey_h

#include <Pegasus/Common/Config.h>
#include <Pegasus/General/Linkage.h>
#include <Pegasus/Common/String.h>
#include <Pegasus/Common/CIMObjectPath.h>

PEGASUS_NAMESPACE_BEGIN

class SubscriptionKey;

struct PEGASUS_GENERAL_LINKAGE SubscriptionKeyEqualFunc
{
    static Boolean equal(const SubscriptionKey& x, const SubscriptionKey& y);
};

struct PEGASUS_GENERAL_LINKAGE SubscriptionKeyHashFunc
{
    static Uint32 hash(const SubscriptionKey& x);
};


/**
    Normalizes the subscription path. Host name will be removed from both
    filter and handler objectPaths. Namespace will also be removed from
    filter or handler if filter's or handler's namespace is same as
    subscription namespace.
*/

class PEGASUS_GENERAL_LINKAGE SubscriptionKey
{

public:
    SubscriptionKey(){};

    SubscriptionKey(const CIMObjectPath & subscription);

    ~SubscriptionKey(){};

    String toString() const;

protected:
    static void _parseObjectName(
        const String & objectName,
        String & name,
        String & ns,
        String & className);

private:
    String handlerName;
    String filterName;
   
    String filterNamespace;
    String handlerNamespace;
    String subscriptionNamespace;

    String filterClassName;
    String handlerClassName;
    String subscriptionClassName;


    friend Boolean SubscriptionKeyEqualFunc::equal(
        const SubscriptionKey&,
        const SubscriptionKey&);

    friend Uint32 SubscriptionKeyHashFunc::hash(const SubscriptionKey&);
};

PEGASUS_GENERAL_LINKAGE Boolean operator==(
    const SubscriptionKey& key1,
    const SubscriptionKey& key2);

PEGASUS_NAMESPACE_END

#endif  /* Pegasus_SubscriptionKey_h */
